@@ -3,7 +3,7 @@
 /***********************************************************************
 *  This code is part of GLPK (GNU Linear Programming Kit).
 *
-*  Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07 Andrew Makhorin,
+*  Copyright (C) 2000, 01, 02, 03, 04, 05, 06, 07, 08 Andrew Makhorin,
 *  Department for Applied Informatics, Moscow Aviation Institute,
 *  Moscow, Russia. All rights reserved. E-mail: <mao@mai2.rcnet.ru>.
 *
@@ -18,7 +18,7 @@
 *  License for more details.
 *
 *  You should have received a copy of the GNU General Public License
-*  along with this program. If not, see <http://www.gnu.org/licenses/>.
+*  along with GLPK. If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/
 
 #ifndef _GLPK_H
@@ -30,7 +30,7 @@ extern "C" {
 
 /* library version numbers: */
 #define GLP_MAJOR_VERSION  4
-#define GLP_MINOR_VERSION  19
+#define GLP_MINOR_VERSION  30
 
 #ifndef _GLP_PROB
 #define _GLP_PROB
@@ -69,11 +69,8 @@ typedef struct { double _prob; } glp_prob;
 #define GLP_OPT         5  /* solution is optimal */
 #define GLP_UNBND       6  /* solution is unbounded */
 
-#ifndef _GLP_ULONG
-#define _GLP_ULONG
-typedef struct { unsigned int lo, hi; } glp_ulong;
-/* 64-bit unsigned integer data type */
-#endif
+typedef struct { int lo, hi; } glp_long;
+/* long integer data type */
 
 #ifndef _GLP_BFCP
 #define _GLP_BFCP
@@ -107,15 +104,16 @@ typedef struct
 #define GLP_MSG_ERR     1  /* warning and error messages only */
 #define GLP_MSG_ON      2  /* normal output */
 #define GLP_MSG_ALL     3  /* full output */
+#define GLP_MSG_DBG     4  /* debug output */
       int meth;            /* simplex method option: */
 #define GLP_PRIMAL      1  /* use primal simplex */
 #define GLP_DUALP       2  /* use dual simplex, if possible */
       int pricing;         /* pricing technique: */
-#define GLP_PT_STD   0x11  /* standard (textbook) */
+#define GLP_PT_STD   0x11  /* standard (Dantzig rule) */
 #define GLP_PT_PSE   0x22  /* projected steepest edge */
       int r_test;          /* ratio test technique: */
 #define GLP_RT_STD   0x11  /* standard (textbook) */
-#define GLP_RT_HAR   0x22  /* Harris' ratio test */
+#define GLP_RT_HAR   0x22  /* two-pass Harris' ratio test */
       double tol_bnd;      /* spx.tol_bnd */
       double tol_dj;       /* spx.tol_dj */
       double tol_piv;      /* spx.tol_piv */
@@ -129,10 +127,65 @@ typedef struct
       double foo_bar[36];  /* (reserved) */
 } glp_smcp;
 
+#ifndef _GLP_TREE
+#define _GLP_TREE
+typedef struct { double _tree; } glp_tree;
+/* branch-and-bound tree */
+#endif
+
+typedef struct
+{     /* integer optimizer control parameters */
+      int msg_lev;         /* message level: */
+#define GLP_MSG_OFF     0  /* no output */
+#define GLP_MSG_ERR     1  /* warning and error messages only */
+#define GLP_MSG_ON      2  /* normal output */
+#define GLP_MSG_ALL     3  /* full output */
+#define GLP_MSG_DBG     4  /* debug output */
+      int br_tech;         /* branching technique: */
+#define GLP_BR_FFV      1  /* first fractional variable */
+#define GLP_BR_LFV      2  /* last fractional variable */
+#define GLP_BR_MFV      3  /* most fractional variable */
+#define GLP_BR_DTH      4  /* heuristic by Driebeck and Tomlin */
+      int bt_tech;         /* backtracking technique: */
+#define GLP_BT_DFS      1  /* depth first search */
+#define GLP_BT_BFS      2  /* breadth first search */
+#define GLP_BT_BLB      3  /* best local bound */
+#define GLP_BT_BPH      4  /* best projection heuristic */
+      double tol_int;      /* mip.tol_int */
+      double tol_obj;      /* mip.tol_obj */
+      int tm_lim;          /* mip.tm_lim (milliseconds) */
+      int out_frq;         /* mip.out_frq (milliseconds) */
+      int out_dly;         /* mip.out_dly (milliseconds) */
+      void (*cb_func)(glp_tree *tree, void *info);
+                           /* mip.cb_func */
+      void *cb_info;       /* mip.cb_info */
+      int cb_size;         /* mip.cb_size */
+      int pp_tech;         /* preprocessing technique: */
+#define GLP_PP_NONE     0  /* disable preprocessing */
+#define GLP_PP_ROOT     1  /* preprocessing only on root level */
+#define GLP_PP_ALL      2  /* preprocessing on all levels */
+      double mip_gap;      /* relative MIP gap tolerance */
+      int mir_cuts;        /* MIR cuts (GLP_ON/GLP_OFF) */
+      int gmi_cuts;        /* Gomory's cuts (GLP_ON/GLP_OFF) */
+      double foo_bar[34];  /* (reserved) */
+#if 1 /* not yet available */
+      char *fn_sol;        /* file name to write solution found */
+#endif
+} glp_iocp;
+
 /* enable/disable flag: */
 #define GLP_ON          1  /* enable something */
 #define GLP_OFF         0  /* disable something */
 
+/* reason codes: */
+#define GLP_IROWGEN  0x01  /* request for row generation */
+#define GLP_IBINGO   0x02  /* better integer solution found */
+#define GLP_IHEUR    0x03  /* request for heuristic solution */
+#define GLP_ICUTGEN  0x04  /* request for cut generation */
+#define GLP_IBRANCH  0x05  /* request for branching */
+#define GLP_ISELECT  0x06  /* request for subproblem selection */
+#define GLP_IPREPRO  0x07  /* request for preprocessing */
+
 /* return codes: */
 #define GLP_EBADB    0x01  /* invalid basis */
 #define GLP_ESING    0x02  /* singular matrix */
@@ -145,6 +198,12 @@ typedef struct
 #define GLP_ETMLIM   0x09  /* time limit exceeded */
 #define GLP_ENOPFS   0x0A  /* no primal feasible solution */
 #define GLP_ENODFS   0x0B  /* no dual feasible solution */
+#define GLP_EROOT    0x0C  /* root LP optimum not provided */
+#define GLP_ESTOP    0x0D  /* search terminated by application */
+
+/* MPS file format: */
+#define GLP_MPS_DECK    1  /* fixed (ancient) */
+#define GLP_MPS_FILE    2  /* free (modern) */
 
 glp_prob *glp_create_prob(void);
 /* create problem object */
@@ -199,6 +258,9 @@ void glp_del_rows(glp_prob *lp, int nrs, const int num[]);
 void glp_del_cols(glp_prob *lp, int ncs, const int num[]);
 /* delete specified columns from problem object */
 
+void glp_erase_prob(glp_prob *lp);
+/* erase problem object content */
+
 void glp_delete_prob(glp_prob *lp);
 /* delete problem object */
 
@@ -352,6 +414,12 @@ int glp_get_num_int(glp_prob *mip);
 int glp_get_num_bin(glp_prob *mip);
 /* retrieve number of binary columns */
 
+int glp_intopt(glp_prob *mip, const glp_iocp *parm);
+/* solve MIP problem with the branch-and-bound method */
+
+void glp_init_iocp(glp_iocp *parm);
+/* initialize integer optimizer control parameters */
+
 int glp_mip_status(glp_prob *mip);
 /* retrieve status of MIP solution */
 
@@ -364,6 +432,24 @@ double glp_mip_row_val(glp_prob *mip, int i);
 double glp_mip_col_val(glp_prob *mip, int j);
 /* retrieve column value (MIP solution) */
 
+int glp_read_sol(glp_prob *lp, const char *fname);
+/* read basic solution from text file */
+
+int glp_write_sol(glp_prob *lp, const char *fname);
+/* write basic solution to text file */
+
+int glp_read_ipt(glp_prob *lp, const char *fname);
+/* read interior-point solution from text file */
+
+int glp_write_ipt(glp_prob *lp, const char *fname);
+/* write interior-point solution to text file */
+
+int glp_read_mip(glp_prob *mip, const char *fname);
+/* read MIP solution from text file */
+
+int glp_write_mip(glp_prob *mip, const char *fname);
+/* write MIP solution to text file */
+
 int glp_bf_exists(glp_prob *lp);
 /* check if the basis factorization exists */
 
@@ -394,19 +480,109 @@ void glp_ftran(glp_prob *lp, double x[]);
 void glp_btran(glp_prob *lp, double x[]);
 /* perform backward transformation (solve system B'*x = b) */
 
+int glp_eval_tab_row(glp_prob *lp, int k, int ind[], double val[]);
+/* compute row of the simplex tableau */
+
+int glp_eval_tab_col(glp_prob *lp, int k, int ind[], double val[]);
+/* compute column of the simplex tableau */
+
+int glp_ios_reason(glp_tree *tree);
+/* determine reason for calling the callback routine */
+
+glp_prob *glp_ios_get_prob(glp_tree *tree);
+/* access the problem object */
+
+void glp_ios_tree_size(glp_tree *tree, int *a_cnt, int *n_cnt,
+      int *t_cnt);
+/* determine size of the branch-and-bound tree */
+
+int glp_ios_curr_node(glp_tree *tree);
+/* determine current active subproblem */
+
+int glp_ios_next_node(glp_tree *tree, int p);
+/* determine next active subproblem */
+
+int glp_ios_prev_node(glp_tree *tree, int p);
+/* determine previous active subproblem */
+
+int glp_ios_up_node(glp_tree *tree, int p);
+/* determine parent subproblem */
+
+int glp_ios_node_level(glp_tree *tree, int p);
+/* determine subproblem level */
+
+double glp_ios_node_bound(glp_tree *tree, int p);
+/* determine subproblem local bound */
+
+int glp_ios_best_node(glp_tree *tree);
+/* find active subproblem with best local bound */
+
+double glp_ios_mip_gap(glp_tree *tree);
+/* compute relative MIP gap */
+
+void *glp_ios_node_data(glp_tree *tree, int p);
+/* access subproblem application-specific data */
+
+int glp_ios_can_branch(glp_tree *tree, int j);
+/* check if can branch upon specified variable */
+
+void glp_ios_branch_upon(glp_tree *tree, int j, int sel);
+/* choose variable to branch upon */
+
+void glp_ios_select_node(glp_tree *tree, int p);
+/* select subproblem to continue the search */
+
+int glp_ios_heur_sol(glp_tree *tree, const double x[]);
+/* provide solution found by heuristic */
+
+void glp_ios_terminate(glp_tree *tree);
+/* terminate the solution process */
+
 const char *glp_version(void);
 /* determine library version */
 
+void glp_term_out(int flag);
+/* enable/disable terminal output */
+
 void glp_term_hook(int (*func)(void *info, const char *s), void *info);
 /* install hook to intercept terminal output */
 
-void glp_mem_usage(int *count, int *cpeak, glp_ulong *total,
-      glp_ulong *tpeak);
+void *glp_malloc(int size);
+/* allocate memory block */
+
+void *glp_calloc(int n, int size);
+/* allocate memory block */
+
+void glp_free(void *ptr);
+/* free memory block */
+
+void glp_mem_usage(int *count, int *cpeak, glp_long *total,
+      glp_long *tpeak);
 /* get memory usage information */
 
 void glp_mem_limit(int limit);
 /* set memory usage limit */
 
+void glp_free_env(void);
+/* free GLPK library environment */
+
+int glp_read_mps(glp_prob *lp, int fmt, const void *parm,
+      const char *fname);
+/* read problem data in MPS format */
+
+int glp_write_mps(glp_prob *lp, int fmt, const void *parm,
+      const char *fname);
+/* write problem data in MPS format */
+
+int glp_read_lp(glp_prob *lp, const void *parm, const char *fname);
+/* read problem data in CPLEX LP format */
+
+int glp_write_lp(glp_prob *lp, const void *parm, const char *fname);
+/* write problem data in CPLEX LP format */
+
+int glp_main(int argc, const char *argv[]);
+/* stand-alone LP/MIP solver */
+
 /**********************************************************************/
 
 #define LPX glp_prob
@@ -516,10 +692,12 @@ void glp_mem_limit(int limit);
 #define LPX_K_BINARIZE  328   /* lp->binarize */
 #define LPX_K_USECUTS   329   /* lp->use_cuts */
 #define LPX_K_BFTYPE    330   /* lp->bfcp->type */
+#define LPX_K_MIPGAP    331   /* lp->mip_gap */
 
 #define LPX_C_COVER     0x01  /* mixed cover cuts */
 #define LPX_C_CLIQUE    0x02  /* clique cuts */
 #define LPX_C_GOMORY    0x04  /* Gomory's mixed integer cuts */
+#define LPX_C_MIR       0x08  /* mixed integer rounding cuts */
 #define LPX_C_ALL       0xFF  /* all cuts */
 
 typedef struct
@@ -1062,6 +1240,10 @@ int lpx_is_b_avail(LPX *lp);
 int lpx_write_pb(LPX *lp, const char *fname, int normalized);
 /* write problem data in (normalized) OPB format */
 
+#define lpx_main _glp_lpx_main
+int lpx_main(int argc, const char *argv[]);
+/* stand-alone LP/MIP solver */
+
 #ifdef __cplusplus
 }
 #endif
